#include <ros/ros.h>
#include <ros/package.h>
#include <tf/transform_broadcaster.h>
#include <move_base_msgs/MoveBaseAction.h>
#include <actionlib/client/simple_action_client.h>

#include <stdio.h>
#include <string.h>

typedef actionlib::SimpleActionClient<move_base_msgs::MoveBaseAction> MoveBaseClient;

int main(int argc, char** argv){
  ros::init(argc, argv, "simple_navigation_goals");

  //tell the action client that we want to spin a thread by default
  MoveBaseClient ac("move_base", true);

  //wait for the action server to come up
  while(!ac.waitForServer(ros::Duration(5.0))){
    ROS_INFO("Waiting for the move_base action server to come up");
  }

  FILE *fd;
  int bulegoa; //Bulego zenbakia
  char izena[50];  //Bulegoa norena den
  char input[3]; //Erabiltzailearen sarrera
  int helburua;  //Erabiltzaileak eskatutako bulego zenbakia
  float helb_x, helb_y, x, y;  //Helburuko koordenatuak
  
  helb_x=0.0;
  helb_y=0.0;
  

  //Bulegoen fitxategia bilatu eta ireki
  std::string path = ros::package::getPath("simple_navigation_goals");
  char *path_string = new char [path.length()+1];
  char *full_path = new char [path.length()+14];
  std::strcpy(path_string, path.c_str());
  sprintf(full_path,"%s/bulegoak.txt",path_string);

  if ((fd=fopen(full_path,"r"))==NULL){
	printf("Errorea! Ezin da aurkitu %s/bulegoak.txt fitxategia.\n\n",full_path);
	return 1;
  }

  printf("\n----- HELBURU BAT BIDALI MARISORGINI -----\n\n");
  printf("Teklatua erabiliz aukeratu bulego edo laborategi bat (idatzi bulego zenbakia).\n");
  printf("Bulego eta laborategiak zerrendatzea nahi baduzu, sakatu z \nIdatzi bulego zenbakia: ");

  //Irakurri bulego zenbakia
  scanf("%s",input);
  printf("\n");

  if(!strcmp(input,"z")){ //z tekla sakatu badu, bulego zerrenda inprimatu
	printf("Bulegoa - Irakaslea\n");
	while(fscanf(fd,"%d %f %f %[^\t\n]s",&bulegoa,&x,&y,izena)!=EOF){
		printf("  %d - %s\n",bulegoa,izena);
	}
  	printf("\nIdatzi bulego zenbakia: ");
        //Irakurri bulego zenbakia
	scanf("%s",input);	
  	fseek(fd,0,SEEK_SET);
  }

  helburua=atoi(input);

  //Eskatutako bulegoaren koordenatuak bilatu fitxategian
  while(fscanf(fd,"%d %f %f %[^\n]s",&bulegoa,&x,&y,izena)!=EOF){
	if(helburua==bulegoa){
		helb_x=x;
		helb_y=y;
		break;
	}
  }
  fclose(fd);

  if(helb_x==0.0 and helb_y==0.0){ //Ez da bulegoa existitzen
	printf("Solairu honetan ez dago %s bulego edo laborategia.\n\n",input);
	return 1;
  }
  else{
	printf("Helburuko posizioa:(%f,%f), %d bulego/laborategia (%s).\n\n",helb_x,helb_y,bulegoa,izena);
  }


  //Helburua robotari bidali
  move_base_msgs::MoveBaseGoal goal; 
  
  goal.target_pose.header.frame_id = "map";
  goal.target_pose.header.stamp = ros::Time::now();

  double yaw = M_PI;

  goal.target_pose.pose.position.x = helb_x;
  goal.target_pose.pose.position.y = helb_y;
  goal.target_pose.pose.orientation = tf::createQuaternionMsgFromYaw(yaw);

  ROS_INFO("Sending goal");
  ac.sendGoal(goal);

  ac.waitForResult();

  if(ac.getState() == actionlib::SimpleClientGoalState::SUCCEEDED){
    ROS_INFO("Destination reached!");
    printf("\nMarisorgin iritsi da %d bulego/laborategira (%s).\n\n",bulegoa,izena);
  }
  else{
    ROS_INFO("The robot failed to reach the destination for some reason.");
    printf("\n!!!!! Marisorgin ezin izan da iritsi %d bulego/laborategira (%s).\n\n",bulegoa,izena);
  }

  return 0;
}
